//
//      Bayonnet.java
//
//      Modified by Y.Motomura(motomura@etl.go.jp)

import java.util.*;
import java.awt.*;
import java.net.URL;
import java.applet.Applet;

public class Bayonnet extends Applet implements TCPserver {
    Field fd;
    FileIO fio;
    ControlPanel cp;
    DisplayPanel dp;

    final static int Default_PORT=6700; /* Base TCP/IP port No.(+NodeId) */
    final static int MOVE = 0;
    final static int LINK = 1;
    final static int UNLINK = 2;

    final static String defaultGraphFile = "graph1.dat";
    static String GraphDirectory = "Graph/";
    int mode = MOVE;
    String graphFile;

    Nodes nodes;
    Edges edges;

    public void init() {
	init(getParameter("filename"));
    }

    public void init(String parameter) {

// Mem space Allocation
        nodes = new Nodes();
        edges = new Edges();

//  GUI section initializing
        mode = MOVE;
        setLayout(new BorderLayout());
        cp = new ControlPanel(this);
        dp = new DisplayPanel(this);
        fd = new Field(this);
        add("North", cp);
        add("Center", fd);
        add("South", dp);

// File loading section
//	graphFile = getParameter("filename");
	graphFile = new String(parameter);
	if(graphFile == null)
		graphFile = GraphDirectory+defaultGraphFile;
        fio = new FileIO(this);
//	load(graphFile);
	dp.init();  // This method should be called after File loading.
        new TCP(Default_PORT,this);  // This port is Main network I/F
    }

    public void start() {
        fd.start();
    }

    public void stop() {
        fd.stop();
    }

    public void quit(){
	System.exit(0);
    }

// real process method service(input, output) used in TCP.class is defined here.
    public void service(String in, StringBuffer out){
	int i;
	StringTokenizer st;
	String word;
	Node n,n1,n2;
//	Edge edge;

	st = new StringTokenizer(in, "( )\"\n\t\r");
	if(st.hasMoreTokens()!=true) return;
	word = new String(st.nextToken());
System.out.println("Message = "+in);
	if("connect".equals(word)) return;
	if("new-edge".equals(word)){
		n1 = nodes.pickUpByName(st.nextToken());
		n2 = nodes.pickUpByName(st.nextToken());
		if (n1 != null || n2 != null) addEdge(n1,n2);
		else System.out.println("Nodes not found.");
		return;
	}
	if("new-node".equals(word)){
		word = st.nextToken();
		n = nodes.pickUpByName(word);
		if(n == null){
		   n = new Node(50+nodes.size(), 50);
		   addNode(n, word, Integer.parseInt(st.nextToken()));
		}else n.init(Integer.parseInt(st.nextToken()));
		return;
	}
	n = nodes.pickUpByName(word);
	if(n != null){
// If return value is required("get"), put Node name at first.
	   if("get".equals(st.nextToken())){
		  out.append("(");
		  out.append(n.name);out.append(" ");
	   }
	   n.service(in.substring(word.length()),out);
	   out.append(")");
	}
    }

    void save() {
	fio.save(nodes, edges);
	return;
   }

    void load() {
	fd.clear();
	fio.load(graphFile);
    }

    void load(String fname) {
	fd.clear();
	graphFile = fname;
	fio.load(fname);
    }

    public void addNode(Node n, String name, int Type){
	   n.name = name;
	   n.Type = Type;
	   addNode(n);
    }

    public void addNode(Node n){
        nodes.addElement(n);
	n.init();
// At the same time, text Field is also generated.
        dp.addNode(n);
	dp.addNewPanel();
    }

    public void addEdge(Node ns, Node ne){
	Edge edge = new Edge(ns,ne);
	edge.reshape();
	edges.addElement(edge);
	edge.fromNode.out.addElement(edge);
	edge.toNode.in.addElement(edge);
	edge.toNode.inNodes.addElement(edge.fromNode);
	edge.toNode.child_f = true;
	return;
    }

    public static void main(String argv[]){
		Frame fm = new Frame("BAYONNET");
		Bayonnet bn = new Bayonnet();
		if(argv.length > 1){
			bn.init(argv[1]);
		}else{
			bn.init(defaultGraphFile);
		}
		bn.start();
		fm.add("Center", bn);
//		fm.resize(760,680);
		fm.resize(760,200);
		fm.show();
    }
}

