// -*- c++ -*-

//
//      (graphical)Node.java
//
//	Programmed by Jun Iio @ Real World Computing Partnership
//		      iiojun@trc.rwcp.or.jp
//      Modified by Y.Motomura(motomura@etl.go.jp)

import java.util.*;
import java.awt.*;

class gNode extends Rectangle {
    final static Color normalColor = Color.white;
    final static Color selectedColor = Color.yellow;
//    final static Color selectedColor = new Color(230, 230, 250);
    final static Color rootColor = Color.red;
    final static Color markedColor = Color.gray.darker();
    final static int defaultX = 0;
    final static int defaultY = 0;
    final static int defaultW = 15;
    final static int defaultH = 20;

/* Node identfier */
    int nodeId;
    String name;

/* Node type +n ... Descrete n dimensional, -n ... Continous n dimensional )*/
    public int Type = 0;          /*  Type of this node */

    boolean selected = false;
    boolean root = false;
    boolean marked = false;

/* difference between dragged point and itsown point
   (the top left-hand corner). */
    int dx, dy;
/* Edge objects which {get toword, go away} this object */
    Edges in, out;


    gNode() {
        this(defaultX, defaultY, defaultW, defaultH);
    }

    gNode(int x, int y) {
	this(x, y, defaultW, defaultH);
    }

    gNode(int x, int y, int w, int h) {
        reshape(x, y, w, h);
        dx = dy = 0;
        in = new Edges();
        out = new Edges();
        nodeId = 0; name="";
    }

    int cx() {	// center X coordinate
	return x + width / 2;
    }

    int cy() {	// center Y coordinate
	return y + height / 2;
    }

    void paint(Graphics g) {
        if (selected) g.setColor(selectedColor);
        else if (root) g.setColor(rootColor);
        else if (marked) g.setColor(markedColor);
        else g.setColor(normalColor);
        g.fillRect(x, y, width, height);
        g.setColor(Color.black);
        g.drawRect(x, y, width, height);
	g.drawString(name, x-height, y+width+20);
	g.drawString(Integer.toString(Type), x+height, y+width+10);
    }

/* "center"
   [return] Point : returns the center point of this rectangle.
                    if this rectangle is empty, this returns null;
 */
    Point center() {
        if (isEmpty()) return null;
        return new Point(x+width/2, y+height/2);
    }

/* "moveEdges"
   move and reshape related (in&out) edges.
 */
    void moveEdges() {
        for (Enumeration e = in.elements(); e.hasMoreElements();) {
            Edge edge = (Edge)e.nextElement();
            edge.to = center();
            edge.reshape();
        }
        for (Enumeration e = out.elements(); e.hasMoreElements();) {
            Edge edge = (Edge)e.nextElement();
            edge.from = center();
            edge.reshape();
        }
    }

/* "unmark"
   Make clean the mark of nodes which are pointed from this node
   (temporary tracing method is depth-first).
 */
    void unmark() {
        if (!marked) return;
        marked = false;
        for (Enumeration e = out.elements(); e.hasMoreElements();) {
            Edge edge = (Edge)e.nextElement();
            edge.marked = false;
        }
    }        

/* "contentsToString"
 * Print contents for debugging
 */
    String contentsToString(int pass) {
	String retStr;
        switch (pass) {
        case 1:
            retStr = Integer.toString(nodeId) + " " + 
                     Integer.toString(x) + " " +
                     Integer.toString(y) + " " +
                     (root ? "t" : "f") + " \"\"\n" ;
            break;
        case 2:
            retStr = Integer.toString(nodeId) + "\nIn:";	
	    for (Enumeration e = in.elements(); e.hasMoreElements();) {
                    Edge edge = (Edge)e.nextElement();
                    retStr += " " + Integer.toString(edge.edgeId);
                }
            retStr += "\nOut:";
	    for (Enumeration e = out.elements(); e.hasMoreElements();) {
                    Edge edge = (Edge)e.nextElement();
                    retStr += " " + Integer.toString(edge.edgeId);
                }
            retStr += "\n";
            break;
        default:
            retStr = "";
        }
	return retStr;
    }
}


